# /tcl/survey-procs.tcl

ad_library {

  Support procs for simple survey module, most important being
  survey_question_add_to_form instead, which uses the form manager to add a question to a form

  @author philg@mit.edu on
  @author teadams@mit.edu
  @author nstrug@arsdigita.com
  @date   February 9, 2000
  @cvs-id survey-simple-defs.tcl,v 1.29.2.5 2000/07/19 20:11:24 seb Exp

}

ad_proc -public get_survey_info { 
    {-survey_id ""} 
    {-section_id ""} 
} {
    creates a tcl array variable named "survey_info" in the caller's environment,
    which contains key/value pairs for all properties of the requested survey.

    If survey_id is passed in, and it's a single-section survey, the 
    section_id will also be looked up and returned in the survey_info array.

    @author luke@museatech.net
    @date 2002-07-24
} {
    upvar survey_info survey_info

    if {[empty_string_p $survey_id]} {
	db_1row lookup_survey_id ""
    }

    db_1row get_info_by_survey_id "" -column_array survey_info

    if {![info exists survey_info(survey_id)]} {
	# survey doesn't exist, caller has to handle this in their
	# own way
	return
    }
    # If it's a single-section survey, look up the section_id
    if {[empty_string_p $section_id] && $survey_info(single_section_p) == "t"} {
	db_1row lookup_single_section_id ""
	set survey_info(section_id) $section_id
    }

    # quote description where needed, name is usually needed quoted
    set survey_info(name_unquoted) $survey_info(name)
    set survey_info(name) [ad_quotehtml $survey_info(name)]
    set survey_info(description_unquoted) $survey_info(description)
    if {$survey_info(description_html_p)=="f"} {
	set survey_info(description) [ad_quotehtml $survey_info(description)]
    }

    # These aren't used anywhere but cause a lot of load to the db, so just kick them out,
    # I think, they are not so useful - Kolja
    # some useful stats about the survey, dotLRN specific for sloanspace
    #if {[apm_package_installed_p dotlrn]} {
    #	set community_id [dotlrn_community::get_community_id_from_url]
    #	set survey_info(eligible) [db_string n_eligible {}]
    #	set survey_info(completed) [db_string n_completed {}]
    #	set survey_info(not_completed) [expr {$survey_info(eligible) - $survey_info(completed)}]
    #    }
}

ad_proc -public get_section_info { 
    {-section_id ""} 
} {
    creates a tcl array variable named "section_info" in the caller's environment,
    which contains key/value pairs for all properties of the requested section.

} {
    upvar section_info section_info

    db_1row get_info_by_section_id "" -column_array section_info
    if {![info exists section_info(survey_id)]} {
	# section doesn't exist, caller has to handle this in their
	# own way
	return
    }

    # quote description where needed, name is usually needed quoted
    set section_info(name_unquoted) $section_info(name)
    set section_info(name) [ad_quotehtml $section_info(name)]
    set section_info(description_unquoted) $section_info(description)
    if {$section_info(description_html_p)=="f"} {
	set section_info(description) [ad_quotehtml $section_info(description)]
    }
}

ad_proc -public survey_section_add_to_form {
    form
    section_id
    {response_id ""}
} {
    adds the questions for the specified section to the specified form. the forms must have been created with ad_form or template::form:create. This uses survey_question_add_to_form, unless the section is a block section
} {

    get_section_info -section_id $section_id

    # create the section
    template::form::section $form "$section_info(name)"

    template::element create $form section_info$section_id \
	-widget inform \
	-label "" \
	-value $section_info(description)

    # Is there a template for the section?
    set template_filename "[template::get_resource_path]/forms/survey/sections/${section_id}.adp"
    if {[file exists $template_filename]} {
	set section_template [template::util::read_file $template_filename]
	uplevel [list set section_template($section_info(name)) $section_template]
    }

    # If this is a block section, generate a block question, else add questions one by one
    if {$section_info(block_section_p)=="t"} {
	# display as block
	# for this, the "options"-parameter has to be built, which is somewhat complicated
	set explanation ""
	set options [list]
	lappend options [db_list_of_lists answer_descriptions ""]
	set all_choices [list]
	set old_block 0
	db_foreach block_choices "" {
	    # replace {xxx.gif} by image inclusion
	    regsub -all "\{(\[^\{\}\]*)\}" $label {<img src=/graphics/survey/\1>} label
	    if {$old_block!=$block_section_id && [llength $all_choices]>0} {
		set old_block $block_section_id
		lappend choice_lists $all_choices
		set all_choices [list $label]
	    } else {
		set old_block $block_section_id
		lappend all_choices $label
	    }
	}
	lappend choice_lists $all_choices
	lappend options [list $explanation $choice_lists]

	set question_list [db_list_of_lists questions ""]
	foreach question $question_list {
	    set q_id [lindex $question 0]
	    set question_text [lindex $question 1]
	    set details [db_list_of_lists question_details ""]
	    set detail_list [list]
	    foreach detail $details {
		set question_id [lindex $detail 0]
		set required_p  [lindex $detail 1]
		set choice_id_list [db_list choice_ids ""]
		lappend detail_list [list "response_to_question.$question_id" $required_p $choice_id_list]
	    }
	    lappend options [list $question_text $detail_list]
	}

	if {![empty_string_p $response_id]} {
	    set responses [db_list checked ""]
	} else {
	    set responses [list]
	}

	template::element::create $form "section$section_id" \
	    -datatype integer \
	    -widget block \
	    -options $options \
	    -value $responses
    } else {
	db_foreach all_questions "" {
	    # add all questions to the form
	    survey_question_add_to_form $form $question_id $response_id
	}
    }
}

ad_proc -public survey_question_add_to_form { 
    form
    question_id 
    {response_id ""} 
} {Adds the question to the form. the form has to be created by ad_form or with template::form::create. The form variable is of the form \"response_to_question.\$question_id} {

    if {![empty_string_p $response_id]} {
	set edit_previous_response_p "t"
    } else {
	set edit_previous_response_p "f"
    }

    set element_name "response_to_question.$question_id"

    db_1row survey_question_properties ""

    set user_value ""

    if {$predefined_question_id!=[db_null]} {
	# question is a predefined question. If it has already been answered, get the last response value from the db
	# if it is a predefined question with action_type="db" get the value from the db instead.
	db_1row predefined_question_data ""
	if {$action_type == "db"} {
	    # select preselected value from the db
	    set user_id [ad_get_user_id]
	    # We have to check for date
	    if {$abstract_data_type == "date"} {
		set user_value [db_string ignore "select to_char($column_name,'YYYY MM DD') from $table_name where $key_name=:user_id" -default ""]
		set date_answer $user_value
	    } else {
		set user_value [db_string ignore "select $column_name from $table_name where $key_name=:user_id" -default ""]
		set choice_id $user_value
		set boolean_answer $user_value
		set clob_answer $user_value
		set number_answer $user_value
		set varchar_answer $user_value
		set attachment_answer $user_value
	    }
	} else {
	    # select preselected value from latest response
	    set user_id [ad_get_user_id]
	    if {[db_0or1row locate_predef_preselect ""]} {
		set count 0
		db_foreach prev_response_query_predef {} {
		    incr count
		    
		    if {$presentation_type == "checkbox"} {
			set selected_choices($choice_id) "t"
		    }
		} if_no_rows {
		    set choice_id 0
		    set boolean_answer ""
		    set clob_answer ""
		    set number_answer ""
		    set varchar_answer ""
		    set date_answer ""
		    set attachment_answer ""
		}
		set edit_previous_response_p "t"
	    } else {    
		set choice_id 0
		set boolean_answer ""
		set clob_answer ""
		set number_answer ""
		set varchar_answer ""
		set date_answer ""
		set attachment_answer ""
	    }
	}
    } else {
	if {$edit_previous_response_p == "t"} {
	    set user_id [ad_get_user_id]
	    
	    set count 0
	    db_foreach prev_response_query {} {
		incr count
		
		if {$presentation_type == "checkbox"} {
		    set selected_choices($choice_id) "t"
		}
	    } if_no_rows {
		set choice_id 0
		set boolean_answer ""
		set clob_answer ""
		set number_answer ""
		set varchar_answer ""
		set date_answer ""
		set attachment_answer ""
	    }
	}
    }

    if {$edit_previous_response_p == "t"} {
	switch -- $abstract_data_type {
	    "choice" {
		set user_value $choice_id
	    }
	    "shorttext" {
		set user_value $varchar_answer
	    }
	    "boolean" {
		set user_value $boolean_answer
            }
	    "integer" -
            "number" {
                set user_value $number_answer
	    }
	    "text" {
                set user_value $clob_answer
	    }
	    "date" {
		set user_value $date_answer
	    }
	    "blob" {
		if {![empty_string_p $attachment_answer]} {
		    set package_id [ad_conn package_id]
		    set filename [db_string get_file_name ""]
		    regsub "_$response_id$" $filename "" filename		   
		    set user_value "Uploaded file: <a href=\"[site_node::get_url_from_object_id -object_id $package_id]/view-attachment?[export_url_vars response_id question_id]\">\"$filename\"</a>"
		}
	    }
	}
    }

    # add the form elements, depending on the presentation type
    switch -- $presentation_type {
        "upload_file"  {
	    template::element::create $form $element_name \
		-widget file \
		-label "$question_text" \
		-required_p $required_p \
		-value $user_value
	}
	"textbox" {
	    set html [ad_decode $presentation_options "large" {size 70} "medium" {size 40} {size 10}]
	    template::element::create $form $element_name \
		-datatype text \
		-widget text \
		-label "$question_text" \
		-value $user_value \
		-html $html \
		-required_p $required_p
	}
	"textarea" {
	    set html [ad_decode $presentation_options "large" {rows 20 cols 65} "medium" {rows 15 cols 55} {rows 8 cols 35}] 
	    template::element::create $form $element_name \
                -datatype text \
                -widget textarea \
                -label "$question_text" \
		-value $user_value \
                -html $html \
		-required_p $required_p
	    }
	"date" {
	    #set options {format "DD Month YYYY"}
            template::element::create $form $element_name \
                -datatype date \
                -widget date \
                -label "$question_text" \
                -value $user_value \
		-required_p $required_p
	}
	"select" {
	    if { $abstract_data_type == "boolean" } {
		if {![empty_string_p $presentation_options]} {
		    set options_list [split $presentation_options "/"]
		    set choice_t [lindex $options_list 0]
		    set choice_f [lindex $options_list 1]
		} else {
		    set choice_t "True"
		    set choice_f "False"
		}
		set options "{$choice_t t} {$choice_f f}"
	    } else {
		# at some point, we may want to add a UI option for the admin
		# to sepcify multiple or not for select
		set optionlist [list]
		db_foreach question_choices "" {
		    lappend optionlist [list $label $choice_id]
		}
		set options $optionlist
	    }
	    template::element::create $form $element_name \
                -datatype text \
                -widget select \
                -label "$question_text" \
                -value $user_value \
                -options $options \
		-required_p $required_p
 	}
 	"radio" {
	    set widget "text(radio)"
	    if { $abstract_data_type == "boolean" } {
		if {![empty_string_p $presentation_options]} {
		    set options_list [split $presentation_options "/"]
		    set choice_t [lindex $options_list 0]
		    set choice_f [lindex $options_list 1]
		} else {
		    set choice_t "True"
		    set choice_f "False"
		}

		set options "{$choice_t t} {$choice_f f}"
	    } else {
		set optionlist [list]
		db_foreach question_choices_2 "" {
		    # replace {xxx.gif} by image inclusion
		    regsub -all "\{(\[^\{\}\]*)\}" $label {<img src=/graphics/survey/\1>} label
		    lappend optionlist [list $label $choice_id]
		}
		set options $optionlist
	    }  
	    template::element::create $form $element_name \
                -datatype text \
                -widget radio \
                -label "$question_text" \
                -value $user_value \
                -options $options \
		-required_p $required_p
	}

	"checkbox" {
	    set choices [list]
	    set optionlist [list]
	    db_foreach question_choices_3 "" {
		lappend optionlist [list $label $choice_id]
	    }
	    set options $optionlist
            template::element::create $form $element_name \
                -datatype text \
                -widget checkbox \
                -label "$question_text" \
                -values $user_value \
                -options $options \
		-required_p $required_p
 	}
    }
}

ad_proc -public util_show_plain_text { text_to_display } "allows plain text (e.g. text entered through forms) to look good on screen without using tags; preserves newlines, angle brackets, etc." {
    regsub -all "\\&" $text_to_display "\\&amp;" good_text
    regsub -all "\>" $good_text "\\&gt;" good_text
    regsub -all "\<" $good_text "\\&lt;" good_text
    regsub -all "\n" $good_text "<br>\n" good_text
    # get rid of stupid ^M's
    regsub -all "\r" $good_text "" good_text
    return $good_text
}

ad_proc -public survey_answer_summary_display {response_id {html_p 1}} "Returns a string with the questions and answers. If html_p =t, the format will be html. Otherwise, it will be text.  If a list of category_ids is provided, the questions will be limited to that set of categories." {

    set return_string ""
    set question_id_previous ""
    
    set last_section_id ""
    db_foreach summary "" {
	if {$section_id != $last_section_id} {
	    append return_string "<h4>[ad_quotehtml $section_name]</h4>"
	    set last_section_id $section_id
	}

	if {$question_id == $question_id_previous} {
	    continue
	}
	
	if $html_p {
	    append return_string "<b># $sort_order: $question_text</b> 
	<blockquote>"
	} else {
	    append return_string "# $sort_order: $question_text:  "
	}
	append return_string [util_show_plain_text "$clob_answer $number_answer $varchar_answer $date_answer"]
	
	if {![empty_string_p $attachment_answer]} {
	    set package_id [ad_conn package_id]
	    set filename [db_string get_filename {}]
	    regsub "_$response_id$" $filename "" filename
	    append return_string "Uploaded file: <a href=\"[site_node::get_url_from_object_id -object_id $package_id]/view-attachment?[export_url_vars response_id question_id]\">\"$filename\"</a>"
	}
	
	if {$choice_id != 0 && ![empty_string_p $choice_id] && $question_id != $question_id_previous} {
	    set label_list [db_list survey_label_list ""]
	    append return_string "[join $label_list ", "]"
	}
	
	if ![empty_string_p $boolean_answer] {
	    append return_string "[survey_decode_boolean_answer -response $boolean_answer -question_id $question_id]"
	    
	}
	
	if $html_p {
	    append return_string "</blockquote>
	    <P>"
	} else {
	    append return_string "\n\n"
	}
	
	set question_id_previous $question_id 
    }
    
    return "$return_string"
}



ad_proc -public survey_get_score {section_id user_id} "Returns the score of the user's most recent response to a survey" {
    
    set response_id [ survey_get_response_id $section_id $user_id ]
    
    if { $response_id != 0 } {
        set score [db_string get_score "" -default 0]
    } else {
        set score {}
    }
    
    return $score
}


ad_proc -public survey_display_types {
} {
    return {list table paragraph}
}


ad_proc -public survey_question_copy {
    {-new_section_id ""}
    {-question_id:required}
} { copies a question within the same survey
} {
    set user_id [ad_conn user_id]
    db_1row get_question_details {}
    if {![empty_string_p $new_section_id]} {
	set section_id $new_section_id
    }

    set old_question_id $question_id
    if {[empty_string_p $new_section_id]} {
	set after $sort_order
	set new_sort_order [expr {$after + 1}]
	    db_dml renumber_sort_orders {}
    } else {
	set new_sort_order $sort_order
    }

    set new_question_id [db_exec_plsql create_question {}]
    db_dml insert_question_text {}
    db_foreach get_survey_question_choices {} {
	set new_choice_id [db_nextval survey_choice_id_sequence]
	db_dml insert_survey_question_choice {}
	
    }

  return $new_question_id
}

ad_proc survey_copy { 
    {-survey_id:required} 
    {-package_id ""} 
    {-new_name ""}
} {
    copies a survey, copying all questions, but not responses
    if package_id is specific it copies the survey to another
    survey package instance, otherwise it copies the survey to the
    same package instance
} {

    if {[empty_string_p $package_id]} {
	set package_id [ad_conn package_id]
    }

    db_1row get_survey_info {}
    if {![empty_string_p $new_name]} {
	set name $new_name
    }
    set user_id [ad_conn user_id]
    set new_survey_id [db_exec_plsql survey_create {} ]
    set sections_list [db_list get_sections {}]


    foreach section_id $sections_list {
	db_1row get_section_info ""
	set new_section_id [db_exec_plsql section_create {}]
	set new_section_ids($section_id) $new_section_id
	if {![empty_string_p $description]} {
	    db_dml set_section_description {}
	}
    }
   db_foreach get_questions {} {

	survey_question_copy -new_section_id $new_section_ids($section_id) -question_id $question_id
    }
return $new_survey_id

}

ad_proc -public survey_do_notifications {
    {-response_id ""}
} { process notifications when someone responds to a survey
    or edits a response
} {

        set survey_id [db_string get_survey_id_from_response {}]
	get_survey_info -survey_id $survey_id
	set survey_name $survey_info(name)
	set subject "Response to $survey_name"

        #dotlrn specific info
    set dotlrn_installed_p [apm_package_installed_p dotlrn]
	if {$dotlrn_installed_p && ![empty_string_p [dotlrn_community::get_community_id]]} {
	set package_id [ad_conn package_id]
	set community_id [dotlrn_community::get_community_id]
	set segment_id [dotlrn_community::get_rel_segment_id -community_id $community_id -rel_type "dotlrn_member_rel"]
	if {![empty_string_p $community_id]} {
	    set community_name [dotlrn_community::get_community_name $community_id]
	} else {
	    set community_name "No Community"
	}
	set community_url "[ad_parameter -package_id [ad_acs_kernel_id] SystemURL][dotlrn_community::get_community_url $community_id]"
    }
	db_1row get_response_info {}
	
	set notif_text ""
	if {$dotlrn_installed_p} {
	    append notif_text "
Group: $community_name"
        }
	append notif_text "
Survey: $survey_name
Respondent: $user_name 

Here is what $user_name <[acs_community_member_url -user_id $responding_user_id]>
had to say in response to $survey_name:
	"

	if {$edit_p} {
	    append notif_text "
Edited "
	}
	append notif_text "Response on $response_date\n"

	append notif_text [survey_answer_summary_display $response_id 0]

# add summary info for sloanspace
	if {$dotlrn_installed_p} {
	set n_responses [db_string n_responses {}]
	if {$n_responses > 0} {
	    append notif_text " -----
Already Responsed: $n_responses users

View these users. <$community_url/survey/admin/respondents?response_type=responded>

Spam these users. <$community_url/survey/admin/send-mail?survey_id=$survey_id&to=responded>

"
        }
	set n_members [db_string n_members {}]
        set n_awaiting [expr {$n_members - $n_responses}]

        append notif_text "
Awaiting a response: $n_awaiting users

View these users. <$community_url/survey/admin/respondents?response_type=not_responded>

Spam these users. <$community_url/survey/admin/send-mail?survey_id=$survey_id&to=not_responded>

The whole group: $n_members

View these users. <$community_url/survey/admin/respondents?response_type=all>

Spam these users. <$community_url/survey/admin/send-mail?survey_id=$survey_id&to=all>

Responses:
"

        db_foreach get_questions {} {
        append notif_text "$sort_order.    $question_text - View responses. <$community_url/survey/view-text-responses?question_id=$question_id>
    "
        }
    }
	notification::new \
            -type_id [notification::type::get_type_id \
			  -short_name survey_response_notif] \
            -object_id $survey_id \
            -response_id $survey_id \
            -notif_subject $subject \
            -notif_text $notif_text
    
}


ad_proc survey_decode_boolean_answer { 
    {-response:required}
    {-question_id:required}
} {
    takes t/f value from a boolean_answer column and 
    decodes it based on the presentation_options of the question

    @author Dave Bauer <dave@thedesignexperience.org>

    @param -response text value of response to be decoded
    @param -question_id question_id of question response is from
} {
    set presentation_options [db_string get_presentation_options {}]
    if {[empty_string_p $presentation_options]} {
	set presentation_options "True/False"
    }


    if {![empty_string_p $response]} {
	set options_list [split $presentation_options "/"]

	if {$response=="t"} {
	    set response [lindex $options_list 0]
	} else {
	    set response [lindex $options_list 1]
	}
    } 
    return $response
}

ad_proc survey_next_sections { 
    {-edit_p "f"}
    survey_id
    section_id
    response_id
} {
    returns the following sections in the survey. First get a list of all sections after the one just completed, then see, if their condition (in case of branching) holds. If yes or if there are no conditions, add the section. Repeat this, until a condition cannot be decided or a section has page_break_p="t"
} {
    if {$section_id==0} {
	# no last section given, get the last one answered or the first one if editing
	if {$edit_p=="t"} {
	    set sort_key 0
	} else {
	    set sort_key [db_string get_last_answered_section_sort_key ""]
	}
    } else {
	set sort_key [db_string sort_key ""]
    }

    set result [list]
    set sections [db_list_of_lists get_following_sections ""]
    foreach section $sections {
	set section_id   [lindex $section 0]
	set page_break_p [lindex $section 1]
	set ok "t"
	db_foreach get_condition "" {
	    if {![db_0or1row get_response ""]} {
		set ok "f"
	    }
	}
	if {$ok=="t"} {
	    lappend result $section_id
	}
	if {$page_break_p=="t" && $result!=[list]} {
	    return $result
	}
    }
    return $result
}

ad_proc survey_status_bar { 
    section_id
} {
    returns a status bar, indicating the progress within a survey
} {
    db_1row percentage ""
    if {[empty_string_p $status_bar_color]} {
	return ""
    }
    set html "<!-- BEGIN status bar - sets slider width to 160 pixels -->
<table width=160 border=0 cellpadding=0 cellspacing=0>
 <tr>
  <td>
<!-- this table is for the verbal display of info -->
  <table width=\"100%\" border=0 cellpadding=0 cellspacing=0>
   <tr>
    <td align=\"center\"><small>$percentage% Complete</small>
    </td>
   </tr>
   <tr>
    <td align=\"center\">
<!-- this table is for the visual display of info, set black box width here -->
    <table width=\"120\" border=0 cellpadding=0 cellspacing=2 bgcolor=\"black\">
     <tr>
      <td bgcolor=\"gray\" align=\"left\">
<!-- this table is for the colored box, image is for Netscape and opera bug  -->
      <table width=\"$percentage%\" border=0 cellpadding=0 cellspacing=0 bgcolor=\"$status_bar_color\">
       <tr>
        <td><img src=\"/images/dot_clear.gif\" width=1 height=13 alt=\"\" border=0>
        </td>
       </tr>
      </table>
<!-- close colored box table -->
     </td>
    </tr>
   </table>
<!-- close colored box table -->
   </td>
  </tr>
 </table>
</td>
</tr>
</table>
<!-- END status bar -->"
    return $html
}

ad_proc survey_block_section_candidate_p { 
    section_id
} {
    return 1 if the section can be made a block section, 0 if not
    criteria: 
    - at least one question
    - All questions are multiple choice, radio button
    - For each question_text there must be the same set(s) of choices
} {
    set return_value 1
    db_foreach get_questions "" {
	if {$abstract_data_type != "choice" || $presentation_type!= "radio"} {
	    set return_value 0
	}
	if {[info exist question($question_text)]} {
	    append question($question_text) ":$all_choices"
	} else {
	    set question($question_text) $all_choices
	}
    } if_no_rows {
	set return_value 0
    }
    set count 0
    foreach name [array names question] {
	if {$count==0} {
	    set answers $question($name)
	} else {
	    if {$question($name)!=$answers} {
		return 0
	    }
	}
	incr count
    }
    return $return_value
}

ad_proc survey_export_csv {
    conn
    args
} {
    Return csv data of a survey to a user, invoked by ns_register_proc from survey-init.tcl
} {
    ad_page_contract {
	
	CSV export of responses to particular survey.
	
	@author  sebastian@arsdigita.com
	@date    July 2000
	@cvs-id  $Id: survey-procs.tcl,v 1.1.2.3 2004/01/13 22:53:06 maltes Exp $
	
    } {
	
	survey_id:integer,notnull
	{anonymous_p:boolean "t"}
	{unique_users_p f}
	on_what_id:optional,integer
	{start:naturalnum 1}
	{end:naturalnum 10000}
    }
    set csv_export ""
    set package_id [ad_conn package_id]
    ad_require_permission $package_id admin
    
    set n_responses [db_string get_n_responses {}]

    if {$n_responses==0} {
	#get_survey_info -survey_id $survey_id
	#set context_bar [ad_context_bar [list "one?[export_url_vars survey_id]" $survey_info(name)] "CSV Export"]
	#_return_template "no-responses"
	ad_return_complaint 1 "No responses"
	return
    }
    
    set question_id_list [list]
    set responses_table survey_responses
    
    set headline "email,first_names,last_name,user_id,submission_date,response_id"
    
    db_foreach get_question_data_types {} {
	lappend question_id_list $question_id
	regsub -all {"} $question_text {""} question_text
        append headline ",\"$question_text"
        append headline "\""
        set question_data_type($question_id) $abstract_data_type
        switch -- $abstract_data_type {
	    "date" {
		set question_column($question_id) "date_answer"
	    }
	    "text" {
		set question_column($question_id) "clob_answer"
	    }
	    "shorttext" {
		set question_column($question_id) "varchar_answer"
	    }
	    "boolean" {
		set question_column($question_id) "boolean_answer"
	    }
	    "integer" -
	    "number" {
		set question_column($question_id) "number_answer"
	    }
	    "choice" {
		set question_column($question_id) "label"
	    }
	    "blob" {
		set question_column($question_id) "attachment_answer"
	    }
	    default {
		set question_column($question_id) "varchar_answer"
	    }
	}
	
    }
    
    #  We're looping over all question responses in survey_question_responses
    
    set current_response_id ""
    set current_response ""
    set current_question_id ""
    set current_question_list [list]
    set csv_export ""
    set r 0
    ReturnHeaders "text/comma-separated-values"
    ns_write "$headline \r\n"
    
    if {$anonymous_p=="t"} {
        set query "get_all_survey_question_responses_with_anonymous"
    } else {
        set query "get_all_survey_question_responses"
    }
    db_foreach $query "" {
	
	if { $response_id != $current_response_id } {
	    if { ![empty_string_p $current_question_id] } {
		append current_response ",\"[join $current_question_list ","]\""
	    }
	    
	    if { ![empty_string_p $current_response_id] } {
		append csv_export "$current_response \r\n"
	    }
	    set current_response_id $response_id
	    set one_response [list $email $first_names $last_name $user_id $creation_date $response_id]
	    regsub -all {"} $one_response {""} one_response
	    set current_response "\"[join $one_response {","}]\""
	    
	    set current_question_id ""
	    set current_question_list [list]
	}
	
	set response_value [set $question_column($question_id)]
	#  Properly escape double quotes to make Excel & co happy
	regsub -all {"} $response_value {""} response_value
	
	#  Remove any CR or LF characters that may be present in text fields
	regsub -all {[\r\n]} $response_value {} response_value
	
	if { $question_id != $current_question_id } {
	    if { ![empty_string_p $current_question_id] } {
		append current_response ",\"[join $current_question_list ","]\""
	    }
	    set current_question_id $question_id
	    set current_question_list [list]
	}
	# decode boolean answers
	if {$question_data_type($question_id)=="boolean"} {
	    set response_value [survey_decode_boolean_answer -response $response_value -question_id $question_id]
	}
	if {$question_data_type($question_id)=="blob"} {
	    set response_value [db_string get_filename {} -default ""]
	}
	lappend current_question_list $response_value
	
	incr r
	if {$r>99} {
	    ns_write "${csv_export} "
	    set csv_export ""
	    set rows 0
	}
	
    }
    
    if { ![empty_string_p $current_question_id] } {
	append current_response ",\"[join $current_question_list ","]\""
    }
    if { ![empty_string_p $current_response_id] } {
	append csv_export "$current_response\r\n"
    }
    if {[empty_string_p $csv_export]} {
	set csv_export "\r\n"
    }
    ns_write $csv_export
    
    ns_conn close
}



